Public Class Form1
    Private m_CopyImage As Bitmap
    Private m_NewImage As Bitmap
    Private m_MasterPassword As String = ""
    Private m_MasterDate As String = ""

    Private Const COL_PASSWORD As Integer = 1
    Private Const COL_COPY As Integer = 3
    Private Const COL_NEW As Integer = 4

    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        ' Restore form positions and data.
        LoadSettings()
    End Sub
    Private Sub LoadSettings()
        ' Get the master password.
        m_MasterPassword = InputBox("Master Password")
        If m_MasterPassword.Length = 0 Then
            Me.Close()
            Exit Sub
        End If

        ' See if we have any data yet.
        Dim crypt_master As String = GetSetting(APP_NAME, "Passwords", "Master", "")
        If crypt_master.Length = 0 Then
            ' This is the first time. Record the master password change date.
            m_MasterDate = Now.ToString()
        Else
            ' Verify that the user entered the correct master password.
            Dim salt_master As String = GetSetting(APP_NAME, "Passwords", "MasterSalt", "")
            Dim plain_master As String = DecryptFromString(crypt_master, m_MasterPassword, salt_master)
            If plain_master <> m_MasterPassword Then
                MessageBox.Show("Incorrect master password", "Incorrect Password", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                m_MasterPassword = ""
                Me.Close()
                Exit Sub
            End If

            Dim crypt_masterdate = GetSetting(APP_NAME, "Passwords", "MasterDate", Now.ToString())
            m_MasterDate = DecryptFromString(crypt_masterdate, m_MasterPassword, salt_master)
        End If

        ' Load the password data.
        For i As Integer = 0 To 1000
            ' Get the next password's name and value.
            Dim crypt_name As String = GetSetting(APP_NAME, "Passwords", "PasswordName" & i.ToString(), "")
            If crypt_name.Length = 0 Then Exit For
            Dim crypt_value As String = GetSetting(APP_NAME, "Passwords", "PasswordValue" & i.ToString(), "")
            Dim salt As String = GetSetting(APP_NAME, "Passwords", "PasswordSalt" & i.ToString(), "")
            Dim crypt_date As String = GetSetting(APP_NAME, "Passwords", "PasswordDate" & i.ToString(), "")

            ' Decrypt the name, password, and date.
            Dim plain_name As String = DecryptFromString(crypt_name, m_MasterPassword, salt)
            Dim plain_value As String = DecryptFromString(crypt_value, m_MasterPassword, salt)
            Dim plain_date As String = DecryptFromString(crypt_date, m_MasterPassword, salt)

            ' Display the result.
            dgvPasswords.Rows.Add(New Object() {plain_name, plain_value, plain_date, Nothing, Nothing})
        Next i

        ' Restore the form and grid settings.
        RestoreFormPosition(Me, APP_NAME, "MainForm")
        RestoreDgvSettings(Me.dgvPasswords, APP_NAME, "MainDgv")
    End Sub

    ' Save form positions and data.
    Private Sub Form1_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
        dgvPasswords.EndEdit()

        SaveSettings()
    End Sub
    Private Sub SaveSettings()
        ' If we didn't get a valid master password, don't save anything.
        If m_MasterPassword.Length = 0 Then Exit Sub

        ' Save the form positions.
        SaveFormPosition(Me, APP_NAME, "MainForm")
        SaveDgvSettings(Me.dgvPasswords, APP_NAME, "MainDgv")

        ' Save the encrypted passwords.
        SaveSetting(APP_NAME, "Passwords", "Test", "Value") ' Just so there's something to delete.
        DeleteSetting(APP_NAME, "Passwords")

        Dim salt_master As String = RandomSalt()
        Dim crypt_master As String = EncryptToString(m_MasterPassword, m_MasterPassword, salt_master)
        Dim crypt_masterdate As String = EncryptToString(m_MasterDate, m_MasterPassword, salt_master)
        SaveSetting(APP_NAME, "Passwords", "Master", crypt_master)
        SaveSetting(APP_NAME, "Passwords", "MasterSalt", salt_master)
        SaveSetting(APP_NAME, "Passwords", "MasterDate", crypt_masterdate)

        ' Save the password data.
        Dim i As Integer
        For Each row As DataGridViewRow In dgvPasswords.Rows
            ' Get the next password's name, value, and date.
            Dim plain_name As String = row.Cells("colName").Value
            Dim plain_value As String = row.Cells("colPassword").Value
            Dim plain_date As String = row.Cells("colChangedDate").Value
            If plain_name Is Nothing Then plain_name = ""
            If plain_value Is Nothing Then plain_value = ""
            If plain_date Is Nothing Then plain_date = ""
            If (plain_name.Length = 0) OrElse (plain_value.Length = 0) OrElse (plain_date.Length = 0) Then
                row.Cells("colName").Value = ""
                row.Cells("colPassword").Value = ""
                row.Cells("colChangedDate").Value = ""
            Else
                ' Encrypt.
                Dim salt As String = RandomSalt()
                Dim crypt_name As String = EncryptToString(plain_name, m_MasterPassword, salt)
                Dim crypt_value As String = EncryptToString(plain_value, m_MasterPassword, salt)
                Dim crypt_date As String = EncryptToString(plain_date, m_MasterPassword, salt)

                ' Save the values.
                SaveSetting(APP_NAME, "Passwords", "PasswordName" & i.ToString(), crypt_name)
                SaveSetting(APP_NAME, "Passwords", "PasswordValue" & i.ToString(), crypt_value)
                SaveSetting(APP_NAME, "Passwords", "PasswordDate" & i.ToString(), crypt_date)
                SaveSetting(APP_NAME, "Passwords", "PasswordSalt" & i.ToString(), salt)
                i += 1
            End If
        Next row
    End Sub

    Private Sub mnuFileExit_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuFileExit.Click
        Me.Close()
    End Sub

    ' Let the user change settings.
    Private Sub mnuFileChangeMasterPassword_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuFileChangeMasterPassword.Click
        Dim frm As New frmChangeMaster
        frm.txtDataFile.Text = m_MasterPassword
        frm.lblLastChanged.Text = "Master Password last changed " & m_MasterDate

        If frm.ShowDialog() = Windows.Forms.DialogResult.OK Then
            ' Save the new master password.
            m_MasterPassword = frm.txtDataFile.Text
            m_MasterDate = Now.ToString()

            ' Re-encrypt the passwords.
            SaveSettings()

            MessageBox.Show("Master password changed. Values saved.", "Changed", MessageBoxButtons.OK, MessageBoxIcon.Information)
        End If
    End Sub

    ' Copy a password to the clipboard or make a new password.
    Private Sub dgvPasswords_CellContentClick(ByVal sender As System.Object, ByVal e As System.Windows.Forms.DataGridViewCellEventArgs) Handles dgvPasswords.CellContentClick
        If e.ColumnIndex = COL_COPY Then
            ' Copy to clipboard.
            Clipboard.Clear()
            Clipboard.SetText(dgvPasswords.Rows(e.RowIndex).Cells(1).Value.ToString())
            Beep()
        ElseIf e.ColumnIndex = COL_NEW Then
            ' Make a new password.
            Dim frm As New frmNewPassword()
            frm.txtPassword.Text = dgvPasswords.Rows(e.RowIndex).Cells(1).Value.ToString()
            If frm.ShowDialog() = Windows.Forms.DialogResult.OK Then
                dgvPasswords.Rows(e.RowIndex).Cells("colPassword").Value = frm.txtPassword.Text
                dgvPasswords.Rows(e.RowIndex).Cells("colChangedDate").Value = Now.ToString()
            End If
        End If
    End Sub

    ' If this is a password, set the new changed date.
    Private Sub dgvPasswords_CellEndEdit(ByVal sender As Object, ByVal e As System.Windows.Forms.DataGridViewCellEventArgs) Handles dgvPasswords.CellEndEdit
        If e.ColumnIndex = COL_PASSWORD Then
            dgvPasswords.Rows(e.RowIndex).Cells("colChangedDate").Value = Now.ToString()
        End If
    End Sub
End Class
