Public Class frmNewPassword
    ' Required implies allowed.
    Private Sub chkRequireLowercase_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles chkRequireLowercase.CheckedChanged
        If chkRequireLowercase.Checked Then chkAllowLowercase.Checked = True
    End Sub
    Private Sub chkRequireUppercase_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles chkRequireUppercase.CheckedChanged
        If chkRequireUppercase.Checked Then chkAllowUppercase.Checked = True
    End Sub
    Private Sub chkRequireNumber_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles chkRequireNumber.CheckedChanged
        If chkRequireNumber.Checked Then chkAllowNumber.Checked = True
    End Sub
    Private Sub chkRequireSpecial_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles chkRequireSpecial.CheckedChanged
        If chkRequireSpecial.Checked Then chkAllowSpecial.Checked = True
    End Sub
    Private Sub chkRequireUnderscore_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles chkRequireUnderscore.CheckedChanged
        If chkRequireUnderscore.Checked Then chkAllowUnderscore.Checked = True
    End Sub
    Private Sub chkRequireSpace_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles chkRequireSpace.CheckedChanged
        If chkRequireSpace.Checked Then chkAllowSpace.Checked = True
    End Sub
    Private Sub chkRequireOther_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles chkRequireOther.CheckedChanged
        If chkRequireOther.Checked Then chkAllowOther.Checked = True
    End Sub

    ' Not allowed implies not required.
    Private Sub chkAllowLowercase_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles chkAllowLowercase.CheckedChanged
        If Not chkAllowLowercase.Checked Then chkRequireLowercase.Checked = False
    End Sub
    Private Sub chkAllowUppercase_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles chkAllowUppercase.CheckedChanged
        If Not chkAllowUppercase.Checked Then chkRequireUppercase.Checked = False
    End Sub
    Private Sub chkAllowNumber_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles chkAllowNumber.CheckedChanged
        If Not chkAllowNumber.Checked Then chkRequireNumber.Checked = False
    End Sub
    Private Sub chkAllowSpecial_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles chkAllowSpecial.CheckedChanged
        If Not chkAllowSpecial.Checked Then chkRequireSpecial.Checked = False
    End Sub
    Private Sub chkAllowUnderscore_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles chkAllowUnderscore.CheckedChanged
        If Not chkAllowUnderscore.Checked Then chkRequireUnderscore.Checked = False
    End Sub
    Private Sub chkAllowSpace_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles chkAllowSpace.CheckedChanged
        If Not chkAllowSpace.Checked Then chkRequireSpace.Checked = False
    End Sub
    Private Sub chkAllowOther_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles chkAllowOther.CheckedChanged
        If Not chkAllowOther.Checked Then chkRequireOther.Checked = False
    End Sub

    ' Generate a new password.
    Private Sub btnGenerate_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnGenerate.Click
        txtPassword.Text = RandomPassword()
    End Sub

    Private Function RandomPassword() As String
        Const LOWER As String = "abcdefghijklmnopqrstuvwxyz"
        Const UPPER As String = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
        Const NUMBER As String = "0123456789"
        Const SPECIAL As String = "~!@#$%^&*():;[]{}<>,.?/\|"
        Dim other As String = txtOther.Text
        If chkRequireOther.Checked AndAlso other.Length < 1 Then
            MessageBox.Show("You cannot require characters from a blank string.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error)
            txtOther.Focus()
            Return txtPassword.Text
        End If

        ' Make a list of allowed characters.
        Dim allowed As String = ""
        If chkAllowLowercase.Checked Then allowed &= LOWER
        If chkAllowUppercase.Checked Then allowed &= UPPER
        If chkAllowNumber.Checked Then allowed &= NUMBER
        If chkAllowSpecial.Checked Then allowed &= SPECIAL
        If chkAllowUnderscore.Checked Then allowed &= "_"
        If chkAllowSpace.Checked Then allowed &= " "
        If chkAllowOther.Checked Then allowed &= other

        ' Pick the number of characters.
        Dim min_chars As Integer = Val(txtMinLength.Text)
        Dim max_chars As Integer = Val(txtMaxLength.Text)
        Dim num_chars As Integer = RandomConstrainedInteger(min_chars, max_chars)

        ' Build the password string.
        Dim password As String = ""
        For i As Integer = 1 To num_chars
            password &= allowed.Substring(RandomConstrainedInteger(0, allowed.Length - 1), 1)
        Next i

        ' Satisfy requirements.
        If chkRequireLowercase.Checked AndAlso password.IndexOfAny(LOWER) = -1 Then
            password = RandomChar(LOWER) & password
        End If
        If chkRequireUppercase.Checked AndAlso password.IndexOfAny(UPPER) = -1 Then
            password = RandomChar(UPPER) & password
        End If
        If chkRequireNumber.Checked AndAlso password.IndexOfAny(LOWER) = -1 Then
            password = RandomChar(NUMBER) & password
        End If
        If chkRequireSpecial.Checked AndAlso password.IndexOfAny(SPECIAL) = -1 Then
            password = RandomChar(SPECIAL) & password
        End If
        If chkRequireUnderscore.Checked AndAlso password.IndexOfAny("_") = -1 Then
            password = "_" & password
        End If
        If chkRequireSpace.Checked AndAlso password.IndexOfAny(" ") = -1 Then
            password = " " & password
        End If
        If chkRequireOther.Checked AndAlso password.IndexOfAny(other) = -1 Then
            password = RandomChar(other) & password
        End If

        ' Trim to the proper length.
        password = password.Substring(0, num_chars)

        ' Randomize (in case we added required characters at the front).
        password = randomizeString(password)

        Return password
    End Function

    ' Return a random character from a string.
    Private Function RandomChar(ByVal str As String) As String
        Return str.Substring(RandomConstrainedInteger(0, str.Length - 1), 1)
    End Function

    ' Return a random permutation of a string.
    Private Function randomizeString(ByVal str As String) As String
        Dim result As String = ""
        For i As Integer = 0 To str.Length - 1
            ' Pick a random character.
            Dim j As Integer = RandomConstrainedInteger(0, str.Length - 1)
            result &= str.Substring(j, 1)
            str = str.Remove(j, 1)
        Next i
        Return result
    End Function
End Class